﻿using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Mvc;
using Model.DTO;
using Services;

namespace API.Controllers
{
    [Route("api/[controller]")]
    [Authorize]
    [ApiController]
    public class LessonsController : Controller
    {
        private IDataService<LessonDTO> _dataService;

        private readonly ILogger<UsersController> _logger;
        public LessonsController(IDataService<LessonDTO> dataService)
        {
            _dataService = dataService;
        }

        [HttpGet("lessons/{page}/{number}")]
        public IActionResult GetLessons(int page, int number)
        {
            var nbLesson = _dataService.LessonDataService.GetLessons(page, number).Count();
            if (nbLesson == 0)
            {
                _logger.LogError("[ERREUR] Aucune leçon trouvée.");
                return StatusCode(204);
            }
            _logger.LogInformation("[INFORMATION] {nb} Leçon(s) trouvée(s)", nbLesson);
            return Ok(_dataService.LessonDataService.GetLessons(page, number));
        }

        [HttpGet("lesson/id/{id}")]
        public IActionResult GetLessonById(int id)
        {
            try
            {
                _logger.LogInformation("[INFORMATION] La leçon avec l'id {id} a été trouvé.", id);
                return Ok(_dataService.LessonDataService.GetLessonById(id));
            }
            catch (ArgumentException)
            {
                _logger.LogError("[ERREUR] Aucune leçon trouvée avec l'id {id}.", id);
                return NotFound();
            }
        }

        [HttpGet("lesson/title/{title}")]
        public IActionResult GetLessonByTitle(string title)
        {
            try
            {
                _logger.LogInformation("[INFORMATION] La leçon avec le titre {title} a été trouvé.", title);
                return Ok(_dataService.LessonDataService.GetLessonByTitle(title));
            }
            catch (ArgumentException)
            {
                _logger.LogError("[ERREUR] Aucune leçon trouvée avec le titre {title}.", title);
                return NotFound();
            }
        }

        [HttpDelete]
        public IActionResult DeleteLesson(int id)
        {
            var success = _dataService.LessonDataService.DeleteLesson(id);
            if (success)
            {
                _logger.LogInformation("[INFORMATION] La leçon avec l'id {id} a été supprimé.", id);
                return Ok(_dataService.LessonDataService.DeleteLesson(id));
            }
            else
            {
                _logger.LogError("[ERREUR] Aucune leçon trouvée avec l'id {id}.", id);
                return NotFound();
            }

        }

        [HttpPost]
        public IActionResult CreateLesson([FromBody] LessonDTO dto)
        {
            if (dto.Title == null || dto.LastPublisher == null)
            {
                return BadRequest();
            }
            _logger.LogInformation("[INFORMATION] Une leçon a été créé : title - {title}, lastPublisher - {publisher}, lastEdit - {lastEdit}", dto.Title, dto.LastPublisher, dto.LastEdit);
            return Created(nameof(GetLessons), _dataService.LessonDataService.CreateLesson(dto.Title, dto.LastPublisher, dto.LastEdit));
        }

        [HttpPut]
        public IActionResult UpdateLesson(int id, [FromBody] LessonDTO lessonDTO)
        {
            if (id != lessonDTO.Id)
            {
                _logger.LogError("[ERREUR] Problème ID - La mise à jour de la leçon avec l'id {id} a échouée.", id);
                return BadRequest();
            }
            if (!ModelState.IsValid)
            {
                _logger.LogError("[ERREUR] Problème controlleur - La mise à jour de la leçon avec l'id {id} a échouée.", id);
                return BadRequest();
            }
            if (lessonDTO != null)
            {
                _logger.LogInformation("[INFORMATION] La mise à jour de la leçon avec l'id {id} a été effectuée", id);
                return Ok(_dataService.LessonDataService.UpdateLesson(id, lessonDTO));
            }
            _logger.LogError("[ERREUR] Aucune leçon trouvée avec l'id {id}.", id);
            return NotFound();
        }
    }
}